<!DOCTYPE html>
<!-- saved from url=(0065)http://twitter.github.io/bootstrap/examples/starter-template.html -->
<html lang="en">
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <meta charset="utf-8">
        <title>{% block "title" %}${long_name}{% endblock %}</title>
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <meta name="description" content="">
        <meta name="author" content="">

        <!-- Le styles -->
        <link href="${.assets_path}css/bootstrap.css" rel="stylesheet">
        <link href="${.assets_path}css/bootstrap-responsive.css" rel="stylesheet">
        <link href="${.assets_path}css/custom.css" rel="stylesheet">

        <style>
        {% emptyblock "css" %}
        </style>
    </head>
    <!-- HTML5 shim, for IE6-8 support of HTML5 elements -->
    <!--[if lt IE 9]>
      <script src="../assets/js/html5shiv.js"></script>
    <![endif]-->

  <body>
    <div id="wrap">
    <div id="main-nav" class="navbar navbar-default navbar-static-top navbar-inverse" role="banner">
        <div class="container">
            <div class="navbar-header">
              <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".doc-navbar-collapse">
                <span class="sr-only">Toggle navigation</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
              </button>
              <a class="navbar-brand" href="${relto:.urls.cover.cover}">${lib.title}</a>
            </div>
            <nav class="collapse navbar-collapse doc-navbar-collapse" role="navigation">
                <ul class="nav navbar-nav">
                    <li {% if .section == 'reference' and .path != .urls.doc.tutorial %}class="active"{% endif %}>
                        <a href="${relto:.urls.doc.index}">Reference</a>
                    </li>
                    {%- if .docs['tags.index']['data']['tags'] %}
                    <li {% if .section == 'tags' %}class="active"{% endif %}>
                        <a href="${relto:.urls.tags_index.tags}">Tags</a>
                    </li>
                    {%- endif %}
                    {%- if .docs['command_index.index']['data']['commands'] %}
                    <li {% if .section == 'commands' %}class="active"{% endif %}>
                        <a href="${relto:.urls.command_index.command_index}">Commands</a>
                    </li>
                    {% endif %}
                </ul>
            </nav>
        </div>
    </div>

    {% emptyblock "body" %}
    <div id="push"></div>

    </div>

    <div id="footer">
      <div class="container">
        <p class="muted credit">Moya is &copy; 2014 <a href="#">Moya Software Foundation</a>.</p>
      </div>
    </div>

    <script src="${.assets_path}js/jquery.js"></script>
    <script src="${.assets_path}js/bootstrap.min.js"></script>

{% emptyblock "js" %}
</body>

</html>
