{%- if field.label %}
    <label {% attrib class=["col-sm-2", "control-label", field.hidelabel ? 'sr-only': None] %} for="${field.id}">${field.label}{% if field.required %}&nbsp;*{% endif %}</label>
{%- else %}
    <div class="col-sm-2"></div>
{%- endif %}
