{%- for field in self.children %}
    {%- if field.is_form_field and field.visible %}
        <p{% if field.errors %} class="has-error"{% endif %}>
            {%- if field.label %}
            <label for="${field.id}">${field.label}</label>
            {%- endif %}
            {%- render field set style="simple" -%}
            {% include "common/helperror.html" from "moya.forms" %}
        </p>
    {%- else %}
        {% render field set style="simple" %}
    {%- endif %}
{% endfor %}
