<!doctype HTML>
<html>
    <head>
        <title>404 - Not Found</title>
        <style>
            body{
            font-family:arial;
            }

        </style>
    </head>
    <body>
    <h1>404 - Not Found</h1>
    <p>The file <strong>${.request.path}</strong> was not found on this server</p>
    </body>
</html>