<div class="moya-widgets-select-container" id="${id}" data-count="${len:self.children}">
    <input style="position:absolute;margin-left:-9999px;" class="${field.class}" name="${name or field.name}" value="${field.value or ''}"/>
    <div class="moya-widgets-select">
    {% for index, child in enumerate:self.children %}
        {% render child with index=index, field=field %}
    {% endfor %}
    </div>
</div>