{% extends "base.html" %}

{% block "body" %}
<div class="row">
    <div class="col-lg-8">
        {% render sections.header %}
        {% render sections.posts %}
    </div>
    <div class="col-lg-4">
        {% render sections.side %}
    </div>
</div>
{% endblock %}