{%- if path %}
<p>
<a href="${url}">${path}</a> (${filesize:filesize})
</p>
{%- endif %}
<input id="${field.id}" name="${field.name}" type="${field.type or 'file'}" {% attrib class=field.class %}{% if field.disabled %} disabled{% endif %}>
